package com.jtxt.demo.controller;

import com.jtxt.demo.model.LearningMaterial;
import com.jtxt.demo.model.MaterialCategory;
import com.jtxt.demo.model.Question;
import com.jtxt.demo.model.QuestionCategory;
import com.jtxt.demo.service.CategoryService;
import com.jtxt.demo.service.QuestionCategoryService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import java.util.List;

@RestController
public class CategoryController {
    @Autowired
    private CategoryService categoryService;
    @Autowired
    private QuestionCategoryService questionCategoryService;

    @GetMapping("/categories")
    public ResponseEntity<?> showCategoryRootLevel() {
        return new ResponseEntity<>(categoryService.getRootMaterialCategories(), HttpStatus.OK);
    }

    @GetMapping("/q-categories")
    public ResponseEntity<?> showQuestionCategoryRootLevel() {
        return new ResponseEntity<>(questionCategoryService.getRootQuestionCategories(), HttpStatus.OK);
    }

    // 展示当前id对应的category和所有其subCategories
    @GetMapping("/categories/{categoryId}")
    public ResponseEntity<?> getCategoryAndNextLevel(@PathVariable String categoryId) {
        MaterialCategory category = categoryService.getCategoryInfoById(categoryId);
        List<MaterialCategory> categories = categoryService.findSubsByParentId(categoryId);
        // add parent to the first place
        categories.add(0, category);
        return new ResponseEntity<>(categories, category == null ? HttpStatus.NOT_FOUND : HttpStatus.OK);
    }

    @GetMapping("/q-categories/{categoryId}")
    public ResponseEntity<?> getQuestionCategoryAndNextLevel(@PathVariable String categoryId) {
        QuestionCategory category = questionCategoryService.getCategoryInfoById(categoryId);
        List<QuestionCategory> categories = questionCategoryService.findSubsByParentId(categoryId);
        // add parent to the first place
        categories.add(0, category);
        return new ResponseEntity<>(categories, category == null ? HttpStatus.NOT_FOUND : HttpStatus.OK);
    }

    @PostMapping("/admin/materialCategories")
    public ResponseEntity<?> createMaterialCategory(@RequestBody MaterialCategory category) {
        if (categoryService.getCategoryInfoByName(category.getName()) != null) {
            return new ResponseEntity<>(HttpStatus.CONFLICT);
        }
        return new ResponseEntity<>(categoryService.createMaterialCategory(category), HttpStatus.OK);
    }
    @PostMapping("/admin/qCategories")
    public ResponseEntity<?> createQuestionCategory(@RequestBody QuestionCategory category) {
        if (questionCategoryService.getCategoryInfoByName(category.getName()) != null) {
            return new ResponseEntity<>(HttpStatus.CONFLICT);
        }
        return new ResponseEntity<>(questionCategoryService.createQuestionCategory(category), HttpStatus.OK);
    }

    @PutMapping("/admin/categories/{categoryId}")
    public ResponseEntity<?> editCategory(@PathVariable String categoryId, @RequestBody MaterialCategory updatedCategory) {
        MaterialCategory categoryInDb = categoryService.getCategoryInfoById(categoryId);
        if (categoryInDb == null) {
            return new ResponseEntity<>(HttpStatus.NOT_FOUND);
        }
        updatedCategory.setId(categoryId);
        return new ResponseEntity<>(categoryService.updateMaterialCategory(updatedCategory), HttpStatus.OK);
    }
    @PutMapping("/admin/q-categories/{categoryId}")
    public ResponseEntity<?> editQuestionCategory(@PathVariable String categoryId, @RequestBody QuestionCategory updatedCategory) {
        QuestionCategory categoryInDb = questionCategoryService.getCategoryInfoById(categoryId);
        if (categoryInDb == null) {
            return new ResponseEntity<>(HttpStatus.NOT_FOUND);
        }
        updatedCategory.setId(categoryId);
        return new ResponseEntity<>(questionCategoryService.updateQuestionCategory(updatedCategory), HttpStatus.OK);
    }

    @DeleteMapping("/admin/categories/{categoryId}")
    public ResponseEntity<?> deleteCategory(@PathVariable String categoryId) {
        MaterialCategory categoryInDb = categoryService.getCategoryInfoById(categoryId);
        if (categoryInDb == null) {
            return new ResponseEntity<>(HttpStatus.NOT_FOUND);
        }
        boolean rst = categoryService.deleteCategory(categoryInDb);
        return new ResponseEntity<>(rst ? HttpStatus.OK : HttpStatus.NOT_ACCEPTABLE);
    }

    @DeleteMapping("/admin/q-categories/{categoryId}")
    public ResponseEntity<?> deleteQuestionCategory(@PathVariable String categoryId) {
        QuestionCategory categoryInDb = questionCategoryService.getCategoryInfoById(categoryId);
        if (categoryInDb == null) {
            return new ResponseEntity<>(HttpStatus.NOT_FOUND);
        }
        boolean rst = questionCategoryService.deleteCategory(categoryInDb);
        return new ResponseEntity<>(rst ? HttpStatus.OK : HttpStatus.NOT_ACCEPTABLE);
    }

    @GetMapping("/categories/{categoryId}/materials")
    public ResponseEntity<?> getAllMaterialsUnderThisCategory(@PathVariable String categoryId) {
        return new ResponseEntity<>(categoryService.showMaterialsInCategory(categoryId), HttpStatus.OK);
    }

    @GetMapping("/q-categories/{categoryId}/questions")
    public ResponseEntity<?> getAllQuestionsUnderThisCategory(@PathVariable String categoryId) {
        return new ResponseEntity<>(questionCategoryService.showQuestionsInCategory(categoryId), HttpStatus.OK);
    }

    @PostMapping("/categories/{categoryId}/materials")
    public ResponseEntity<?> addMaterialToCategory(@PathVariable String categoryId, @RequestBody LearningMaterial material) {
        MaterialCategory categoryInDb = categoryService.getCategoryInfoById(categoryId);
        if (categoryInDb == null) {
            return new ResponseEntity<>(HttpStatus.NOT_FOUND);
        }
        boolean rst = categoryService.addMaterialToCategory(material, categoryInDb);
        return new ResponseEntity<>(rst ? HttpStatus.OK : HttpStatus.NOT_ACCEPTABLE);
    }

    @PostMapping("/q-categories/{categoryId}/questions")
    public ResponseEntity<?> addQuestionToCategory(@PathVariable String categoryId, @RequestBody Question question) {
        QuestionCategory categoryInDb = questionCategoryService.getCategoryInfoById(categoryId);
        if (categoryInDb == null) {
            return new ResponseEntity<>(HttpStatus.NOT_FOUND);
        }
        boolean rst = questionCategoryService.addQuestionToCategory(question, categoryInDb);
        return new ResponseEntity<>(rst ? HttpStatus.OK : HttpStatus.NOT_ACCEPTABLE);
    }

    @DeleteMapping("/categories/{categoryId}/materials")
    public ResponseEntity<?> removeMaterialFromCategory(@PathVariable String categoryId, @RequestBody LearningMaterial material) {
        MaterialCategory categoryInDb = categoryService.getCategoryInfoById(categoryId);
        if (categoryInDb == null) {
            return new ResponseEntity<>(HttpStatus.NOT_FOUND);
        }
        boolean rst = categoryService.removeMaterialFromCategory(material, categoryInDb);
        return new ResponseEntity<>(rst ? HttpStatus.OK : HttpStatus.NOT_ACCEPTABLE);
    }
    @DeleteMapping("/q-categories/{categoryId}/questions")
    public ResponseEntity<?> removeQuestionFromCategory(@PathVariable String categoryId, @RequestBody Question question) {
        QuestionCategory categoryInDb = questionCategoryService.getCategoryInfoById(categoryId);
        if (categoryInDb == null) {
            return new ResponseEntity<>(HttpStatus.NOT_FOUND);
        }
        boolean rst = questionCategoryService.removeQuestionFromCategory(question, categoryInDb);
        return new ResponseEntity<>(rst ? HttpStatus.OK : HttpStatus.NOT_ACCEPTABLE);
    }
}
