package com.jtxt.demo.controller;

import com.jtxt.demo.model.*;
import com.jtxt.demo.service.EventService;
import com.jtxt.demo.service.ExamService;
import com.jtxt.demo.service.QuestionService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import java.util.List;

@RestController
public class ExamController {
    @Autowired
    private ExamService examService;
    @Autowired
    private QuestionService questionService;
    @Autowired
    private EventService eventService;
    private static final String DAILY_QUESTIONS = "daily-questions";
    private static final int DAILY_QUESTIONS_NUM = 5;
    private static final String DAILY_POINTS_DESCRIPTION = "通过每日答题所得";

    @PostMapping("/admin/exams")
    public ResponseEntity<?> createExam(@RequestBody Exam exam) {
        Exam newExam = examService.createExam(exam);
        return new ResponseEntity<>(newExam, HttpStatus.OK);
    }
    @GetMapping("/exams/{examId}")
    public ResponseEntity<?> getExam(@PathVariable String examId) {

        Exam exam = examService.getExamById(examId);
        if (exam == null) {
            return new ResponseEntity<>(HttpStatus.NOT_FOUND);
        }
        return new ResponseEntity<>(exam, HttpStatus.OK);
    }
    @PutMapping("/exams/{examId}")
    public ResponseEntity<?> editExam(@PathVariable String examId, @RequestBody Exam updatedExam) {
        Exam exam = examService.getExamById(examId);
        if (exam == null) {
            return new ResponseEntity<>(HttpStatus.NOT_FOUND);
        }
        updatedExam.setId(examId);
        return new ResponseEntity<>(examService.editExam(updatedExam), HttpStatus.OK);
    }
    @DeleteMapping("/exams/{examId}")
    public ResponseEntity<?> deleteExam(@PathVariable String examId) {
        Exam exam = examService.getExamById(examId);
        if (exam == null) {
            return new ResponseEntity<>(HttpStatus.NOT_FOUND);
        }
        return new ResponseEntity<>(examService.deleteExam(exam), HttpStatus.OK);
    }
    @PostMapping("/exams/{examId}/start")
    public ResponseEntity<?> startExam(@PathVariable String examId, @RequestBody User user) {
        if (examId.equals(DAILY_QUESTIONS)) {
            return new ResponseEntity<>(examService.randomlyPickQuestions(DAILY_QUESTIONS_NUM), HttpStatus.OK);
        }
        Exam exam = examService.getExamById(examId);
        if (exam == null) {
            return new ResponseEntity<>(HttpStatus.NOT_FOUND);
        }

        return new ResponseEntity<>(examService.startExam(exam, user), HttpStatus.OK);
    }
    @PostMapping("/exams/{examId}/end")
    public ResponseEntity<?> endExam(@PathVariable String examId, @RequestBody ExamHistory examHistory) {
        if (examId.equals(DAILY_QUESTIONS)) {
            boolean isSuccess = eventService.handleEventWithCustomPoints(examHistory.getUserId(),
                    Event.DAILY_QUESTIONS,
                    examHistory.getPoints(),
                    DAILY_POINTS_DESCRIPTION);
            return new ResponseEntity<>(isSuccess ? HttpStatus.OK : HttpStatus.NOT_ACCEPTABLE);
        }
        Exam exam = examService.getExamById(examId);
        if (exam == null) {
            return new ResponseEntity<>(HttpStatus.NOT_FOUND);
        }
        examHistory.setExamId(examId);
        return new ResponseEntity<>(examService.endExam(examHistory), HttpStatus.OK);
    }
    @GetMapping("/exams")
    public ResponseEntity<?> showAllExams(@RequestBody(required = false) User user) {
        if (user == null) {
            return new ResponseEntity<>(examService.showAllExams(), HttpStatus.OK);
        } else {
             return new ResponseEntity<>(examService.showAllAvailableExams(user), HttpStatus.OK);
        }
    }
    @GetMapping("/exams/history")
    public ResponseEntity<?> showExamsHistory(@RequestParam("user") String userName) {
        if (userName == null) {
            return new ResponseEntity<>(HttpStatus.NOT_FOUND);
        } else {
            return new ResponseEntity<>(examService.showExamHistoriesForUser(userName), HttpStatus.OK);
        }
    }
    @GetMapping("/questions/{questionId}")
    public ResponseEntity<?> getQuestion(@PathVariable String questionId) {
        Question q = questionService.findQuestionById(questionId);
        if (q == null) {
           return new ResponseEntity<>(HttpStatus.NOT_FOUND);
        }
        return new ResponseEntity<>(q, HttpStatus.OK);
    }
    @PostMapping("/questions/")
    public ResponseEntity<?> createQuestion(@RequestBody Question question) {
        return new ResponseEntity<>(questionService.createQuestion(question), HttpStatus.OK);
    }
    @PutMapping("/questions/{questionId}")
    public ResponseEntity<?> editQuestion(@PathVariable String questionId, @RequestBody Question updatedQuestion) {
        Question q = questionService.findQuestionById(questionId);
        if (q == null) {
            return new ResponseEntity<>(HttpStatus.NOT_FOUND);
        }
        updatedQuestion.setId(q.getId());
        return new ResponseEntity<>(questionService.editQuestion(updatedQuestion), HttpStatus.OK);
    }
    @DeleteMapping("/questions/{questionId}")
    public ResponseEntity<?> deleteQuestion(@PathVariable String questionId) {
        Question q = questionService.findQuestionById(questionId);
        if (q == null) {
            return new ResponseEntity<>(HttpStatus.NOT_FOUND);
        }
        return new ResponseEntity<>(questionService.deleteQuestion(q), HttpStatus.OK);
    }
    @PostMapping("/exams/{examId}/questions")
    public ResponseEntity<?> addQuestionsToExam(@PathVariable String examId, @RequestBody List<ExamToQuestion> examToQuestions) {
        Exam exam = examService.getExamById(examId);
        if (examService.addQuestionsToExam(exam, examToQuestions)) {
            return new ResponseEntity<>(HttpStatus.OK);
        } else {
            return new ResponseEntity<>(HttpStatus.NOT_ACCEPTABLE);
        }
    }
    @DeleteMapping("/exams/{examId}/questions")
    public ResponseEntity<?> removeQuestionsToExam(@PathVariable String examId, @RequestBody Question question) {
        Exam exam = examService.getExamById(examId);
        if (examService.removeQuestionFromExam(exam, question)) {
            return new ResponseEntity<>(HttpStatus.OK);
        } else {
            return new ResponseEntity<>(HttpStatus.NOT_ACCEPTABLE);
        }
    }

}
