package com.jtxt.demo.controller;

import com.jtxt.demo.model.Group;
import com.jtxt.demo.model.User;
import com.jtxt.demo.repository.GroupRepository;
import com.jtxt.demo.service.GroupService;
import com.jtxt.demo.service.UserService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import java.util.List;

@RestController
public class GroupController {
    @Autowired
    private GroupService groupService;
    @Autowired
    private UserService userService;

    @GetMapping("/groups/{groupId}")
    public ResponseEntity<?> getGroup(@PathVariable String groupId) {
        Group group = groupService.getGroupById(groupId);
        if (group == null) {
            return new ResponseEntity<>(HttpStatus.NOT_FOUND);
        }
        return new ResponseEntity<>(group, HttpStatus.OK);
    }
    @GetMapping("/groups")
    public ResponseEntity<?> getGroupByName(@RequestParam(value = "group", required = false) String groupName,
                                            @RequestParam(value = "user", required = false) String userName) {
        if (groupName != null) {
            Group groupInDb = groupService.findByName(groupName);
            if (groupInDb == null) {
                return new ResponseEntity<>(HttpStatus.NOT_FOUND);
            }
            return new ResponseEntity<>(groupInDb, HttpStatus.OK);
        }
        if (userName != null) {
            List<Group> groups = groupService.findAllGroupsByUserName(userName);
            return new ResponseEntity<>(groups, HttpStatus.OK);
        }
        return new ResponseEntity<>(HttpStatus.NOT_FOUND);
    }
    @PostMapping("/groups")
    public ResponseEntity<?> createGroup(@RequestBody Group group) {
        return new ResponseEntity<>(groupService.createGroup(group), HttpStatus.OK);
    }
    @PutMapping("/groups/{groupId}")
    public ResponseEntity<?> editGroup(@PathVariable String groupId, @RequestBody Group updatedGroup) {
        Group group = groupService.getGroupById(groupId);
        if (group == null) {
            return new ResponseEntity<>(HttpStatus.NOT_FOUND);
        }
        updatedGroup.setId(groupId);
        return new ResponseEntity<>(groupService.editGroup(updatedGroup), HttpStatus.OK);
    }
    @DeleteMapping("/admin/groups/{groupId}")
    public ResponseEntity<?> deleteGroup(@PathVariable String groupId) {
        Group group = groupService.getGroupById(groupId);
        if (group == null) {
            return new ResponseEntity<>(HttpStatus.NOT_FOUND);
        }
        return new ResponseEntity<>(groupService.deleteGroup(group), HttpStatus.OK);
    }
    @PostMapping("/admin/groups/{groupId}/users")
    public ResponseEntity<?> addUserToGroup(@PathVariable String groupId, @RequestBody User user) {
        Group group = groupService.getGroupById(groupId);
        if (userService.addUserToGroup(user, group)) {
            return new ResponseEntity<>(HttpStatus.OK);
        } else {
            return new ResponseEntity<>(HttpStatus.NOT_ACCEPTABLE);
        }
    }
    @DeleteMapping("/admin/groups/{groupId}/users")
    public ResponseEntity<?> removeUserFromGroup(@PathVariable String groupId, @RequestBody User user) {
        Group group = groupService.getGroupById(groupId);
        if (userService.removeUserFromGroup(user, group)) {
            return new ResponseEntity<>(HttpStatus.OK);
        } else {
            return new ResponseEntity<>(HttpStatus.NOT_ACCEPTABLE);
        }
    }

    @GetMapping("/groups/chain")
    public ResponseEntity<?> getGroupChainById(@RequestParam("group") String groupId) {
        Group group = groupService.getGroupById(groupId);
        if (group == null) {
            return new ResponseEntity<>(HttpStatus.NOT_FOUND);
        }
        return new ResponseEntity<>(groupService.findGroupChainByGroupId(groupId), HttpStatus.OK);
    }

}
