package com.jtxt.demo.controller;

import com.jtxt.demo.model.Comment;
import com.jtxt.demo.model.LearningMaterial;
import com.jtxt.demo.model.User;
import com.jtxt.demo.service.LearningMaterialService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

@RestController
public class LearningMaterialController {
    @Autowired
    LearningMaterialService materialService;

    @GetMapping("/materials/{materialId}")
    public ResponseEntity<?> getMaterial(@PathVariable String materialId) {
        LearningMaterial material = materialService.getMaterialById(materialId);
        if (material == null) {
            return new ResponseEntity<>(HttpStatus.NOT_FOUND);
        }
        return new ResponseEntity<>(material, HttpStatus.OK);
    }
    @PutMapping("/admin/materials/{materialId}")
    public ResponseEntity<?> editMaterial(@PathVariable String materialId, @RequestBody LearningMaterial updatedMaterial) {
        LearningMaterial material = materialService.getMaterialById(materialId);
        if (material == null) {
            return new ResponseEntity<>(HttpStatus.NOT_FOUND);
        }
        updatedMaterial.setId(materialId);
        return new ResponseEntity<>(materialService.editMaterial(updatedMaterial), HttpStatus.OK);
    }
    @PostMapping("/admin/materials")
    public ResponseEntity<?> createMaterial(@RequestBody LearningMaterial newMaterial) {
        return new ResponseEntity<>(materialService.createMaterial(newMaterial), HttpStatus.OK);
    }

    @PostMapping("/materials/{materialId}/read")
    public ResponseEntity<?> readMaterial(@PathVariable String materialId, @RequestBody User user) {
        LearningMaterial material = materialService.getMaterialById(materialId);
        if (material == null) {
            return new ResponseEntity<>(HttpStatus.NOT_FOUND);
        }
        materialService.readMaterial(material, user);
        return new ResponseEntity<>(HttpStatus.OK);
    }
    @GetMapping("/materials/{materialId}/read")
    public ResponseEntity<?> isMaterialReable(@PathVariable String materialId, @RequestParam("user") String userName) {
        LearningMaterial material = materialService.getMaterialById(materialId);
        if (material == null) {
            return new ResponseEntity<>(HttpStatus.NOT_FOUND);
        }
        return new ResponseEntity<>(materialService.isMaterialReadable(material, userName), HttpStatus.OK);
    }
    @GetMapping("/materials/{materialId}/like")
    public ResponseEntity<?> isMaterialLikeable(@PathVariable String materialId, @RequestParam("user") String userName) {
        LearningMaterial material = materialService.getMaterialById(materialId);
        if (material == null) {
            return new ResponseEntity<>(HttpStatus.NOT_FOUND);
        }
        return new ResponseEntity<>(materialService.isMaterialLikeable(material, userName), HttpStatus.OK);
    }
    @PostMapping("/materials/{materialId}/like")
    public ResponseEntity<?> likeMaterial(@PathVariable String materialId, @RequestBody User user) {
        LearningMaterial material = materialService.getMaterialById(materialId);
        if (material == null) {
            return new ResponseEntity<>(HttpStatus.NOT_FOUND);
        }
        materialService.likeMaterial(material, user, true);
        return new ResponseEntity<>(HttpStatus.OK);
    }
    @PostMapping("/materials/{materialId}/dislike")
    public ResponseEntity<?> dislikeMaterial(@PathVariable String materialId, @RequestBody User user) {
        LearningMaterial material = materialService.getMaterialById(materialId);
        if (material == null) {
            return new ResponseEntity<>(HttpStatus.NOT_FOUND);
        }
        materialService.likeMaterial(material, user, false);
        return new ResponseEntity<>(HttpStatus.OK);
    }

    @PostMapping("/materials/{materialId}/comment")
    public ResponseEntity<?> commentMaterial(@PathVariable String materialId, @RequestBody Comment comment) {
        LearningMaterial material = materialService.getMaterialById(materialId);
        if (material == null) {
            return new ResponseEntity<>(HttpStatus.NOT_FOUND);
        }
        comment.setMaterialId(materialId);
        materialService.commentMaterial(comment);
        return new ResponseEntity<>(HttpStatus.OK);
    }

    @GetMapping("/materials/{materialId}/comments")
    public ResponseEntity<?> showAllComments(@PathVariable String materialId) {
        LearningMaterial material = materialService.getMaterialById(materialId);
        if (material == null) {
            return new ResponseEntity<>(HttpStatus.NOT_FOUND);
        }
        return new ResponseEntity<>(materialService.showComments(material), HttpStatus.OK);
    }

    @DeleteMapping("/admin/materials/{materialId}")
    public ResponseEntity<?> deleteMaterial(@PathVariable String materialId) {
        LearningMaterial material = materialService.getMaterialById(materialId);
        if (material == null) {
            return new ResponseEntity<>(HttpStatus.NOT_FOUND);
        }
        boolean rst = materialService.deleteMaterial(material);
        return new ResponseEntity<>(rst ? HttpStatus.OK : HttpStatus.NOT_ACCEPTABLE);
    }

}
