package com.jtxt.demo.service;

import com.jtxt.demo.model.CategoryToMaterial;
import com.jtxt.demo.model.LearningMaterial;
import com.jtxt.demo.model.MaterialCategory;
import com.jtxt.demo.repository.CategoryRepository;
import com.jtxt.demo.repository.CategoryToMaterialRepository;
import com.jtxt.demo.repository.LearningMaterialRepository;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.mongodb.core.MongoTemplate;
import org.springframework.data.mongodb.core.query.Criteria;
import org.springframework.data.mongodb.core.query.Query;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;

@Service
public class CategoryServiceImpl implements CategoryService {
    Logger logger = LoggerFactory.getLogger(CategoryServiceImpl.class);
    @Autowired
    CategoryRepository categoryRepository;
    @Autowired
    MongoTemplate mongoTemplate;
    @Autowired
    CategoryToMaterialRepository categoryToMaterialRepository;
    @Autowired
    LearningMaterialRepository learningMaterialRepository;

    @Override
    public List<LearningMaterial> showMaterialsInCategory(String id) {
        List<CategoryToMaterial> mappings = categoryToMaterialRepository.findAllByCategoryId(id);
        return mappings.stream()
                .map(m -> learningMaterialRepository.findById(m.getMaterialId()))
                .filter(Optional::isPresent)
                .map(Optional::get)
                .collect(Collectors.toList());
    }

    @Override
    public MaterialCategory getCategoryInfoByName(String name) {
        return categoryRepository.findMaterialCategoryByName(name);
    }

    @Override
    public MaterialCategory getCategoryInfoById(String id) {
        return categoryRepository.findById(id).orElse(null);
    }

    @Override
    public List<MaterialCategory> findSubsByParentId(String id) {
        return categoryRepository.findByParentId(id);
    }

    @Override
    public List<MaterialCategory> getRootMaterialCategories() {
        return mongoTemplate.find(Query.query(Criteria.where("parentId").is(null)), MaterialCategory.class);
    }

    @Override
    public MaterialCategory createMaterialCategory(MaterialCategory materialCategory) {
        return categoryRepository.insert(materialCategory);
    }

    @Override
    public boolean deleteCategory(MaterialCategory category) {
        categoryRepository.delete(category);
        return true;
    }

    @Override
    public MaterialCategory updateMaterialCategory(MaterialCategory category) {
        MaterialCategory categoryInDb = getCategoryInfoById(category.getId());
        if (category.getName() != null) {
            categoryInDb.setName(category.getName());
        }
        if (category.getParentId() != null) {
            categoryInDb.setParentId(category.getParentId());
        }
        return categoryRepository.save(categoryInDb);
    }

    @Override
    public boolean addMaterialToCategory(LearningMaterial material, MaterialCategory category) {
        CategoryToMaterial mapping = new CategoryToMaterial();
        mapping.setCategoryId(category.getId());
        mapping.setMaterialId(material.getId());
        categoryToMaterialRepository.insert(mapping);
        return true;
    }

    @Override
    public boolean removeMaterialFromCategory(LearningMaterial material, MaterialCategory category) {
        CategoryToMaterial categoryToMaterial = categoryToMaterialRepository
                .findByCategoryIdAndMaterialId(category.getId(), material.getId());
        categoryToMaterialRepository.delete(categoryToMaterial);
        return true;
    }
}
