package com.jtxt.demo.service;

import com.jtxt.demo.model.Event;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

@Service
public class EventServiceImpl implements EventService {
    @Autowired
    private PointsHistoryService pointsHistoryService;
    @Autowired
    private UserService userService;
    /**
     * 如果以后成为瓶颈的话，需要将逻辑改进为异步且添加中间件MQ. 目前就采取同步的方式
     * @param event
     * @return
     */
    @Override
    public boolean handleEvent(String userName, Event event) {
        if (!isEventValid(event)) return false;
        int deltaPoints = getDefaultPoints(event);
        if (pointsHistoryService.updateUserPoints(userName, event, deltaPoints, null)) {
            return userService.updateUserPoints(userName, deltaPoints);
        }
        return false;
    }

    private int getDefaultPoints(Event event) {
        switch (event) {
            case LOGIN:
            case READ_5_MINS:
                return 1;
            case READ_2_HOURS:
                return 15;
            case DAILY_QUESTIONS:
            case LEARNING_COMPLETE:
                return 2;
        }
        return 0;
    }

    @Override
    public boolean handleEventWithCustomPoints(String userName, Event event, int points, String description) {
        if (description == null) return false; // 必须要说明custom points的描述
        if (!isEventValid(event)) return false;
        if (pointsHistoryService.updateUserPoints(userName, event, points, description)) {
            return userService.updateUserPoints(userName, points);
        }
        return false;
    }

    private boolean isEventValid(Event event) {
        for (Event e : Event.values()) {
            if (e.equals(event)) {
                return true;
            }
        }
        return false;
    }
}
