package com.jtxt.demo.service;

import com.jtxt.demo.model.Group;
import com.jtxt.demo.model.GroupToUser;
import com.jtxt.demo.repository.GroupRepository;
import com.jtxt.demo.repository.GroupToUserRepository;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.stream.Collectors;

@Service
public class GroupServiceImpl implements GroupService {
    @Autowired
    private GroupRepository groupRepository;
    @Autowired
    private GroupToUserRepository groupToUserRepository;
    @Override
    public Group createGroup(Group group) {
        return groupRepository.insert(group);
    }

    @Override
    public Group findByName(String name) {
        return groupRepository.findByName(name);
    }

    @Override
    public Group editGroup(Group group) {
        Group groupInDb = groupRepository.findById(group.getId()).orElse(null);
        if (group.getName() != null) {
            groupInDb.setName(group.getName());
        }
        if (group.getParentGroupId() != null) {
            groupInDb.setParentGroupId(group.getParentGroupId());
        }
        return groupRepository.save(groupInDb);
    }

    @Override
    public Group getGroupById(String id) {
        return groupRepository.findById(id).orElse(null);
    }

    @Override
    public boolean deleteGroup(Group group) {
        groupRepository.delete(group);
        return true;
    }

    @Override
    public List<Group> showAllSubGroups(String parentId) {
        return groupRepository.findAllByParentGroupId(parentId);
    }

    @Override
    public List<Group> findAllGroupsByUserName(String userName) {
        List<GroupToUser> groupList = groupToUserRepository.findAllByUserId(userName);
        return groupList.stream().map(groupToUser -> groupRepository
                .findById(groupToUser.getGroupId()))
                .filter(Optional::isPresent)
                .map(Optional::get)
                .collect(Collectors.toList());
    }

    @Override
    public List<Group> findGroupChainByGroupId(String groupId) {
        Group group = getGroupById(groupId);
        if (group == null) {
            return Collections.emptyList();
        }
        List<Group> rst = new LinkedList<>();
        rst.add(group);
        Group parent = getGroupById(group.getParentGroupId());
        while (parent != null) {
            rst.add(0, parent);
            parent = getGroupById(parent.getParentGroupId());
        }
        return rst;
    }
}
