package com.jtxt.demo.service;

import com.jtxt.demo.model.CategoryToQuestion;
import com.jtxt.demo.model.MaterialCategory;
import com.jtxt.demo.model.Question;
import com.jtxt.demo.model.QuestionCategory;
import com.jtxt.demo.repository.CategoryToQuestionRepository;
import com.jtxt.demo.repository.QuestionCategoryRepository;
import com.jtxt.demo.repository.QuestionRepository;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.mongodb.core.MongoTemplate;
import org.springframework.data.mongodb.core.query.Criteria;
import org.springframework.data.mongodb.core.query.Query;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;

@Service
public class QuestionCategoryServiceImpl implements QuestionCategoryService {
    @Autowired
    private QuestionCategoryRepository questionCategoryRepository;
    @Autowired
    private CategoryToQuestionRepository categoryToQuestionRepository;
    @Autowired
    private QuestionRepository questionRepository;
    @Autowired
    MongoTemplate mongoTemplate;
    @Override
    public List<Question> showQuestionsInCategory(String id) {
        List<CategoryToQuestion> mappings = categoryToQuestionRepository.findAllByCategoryId(id);
        return mappings.stream()
                .map(m -> questionRepository.findById((m.getQuestionId())))
                .filter(Optional::isPresent)
                .map(Optional::get)
                .collect(Collectors.toList());
    }

    @Override
    public QuestionCategory getCategoryInfoByName(String name) {
        return questionCategoryRepository.findQuestionCategoryByName(name);
    }

    @Override
    public QuestionCategory getCategoryInfoById(String id) {
        return questionCategoryRepository.findById(id).orElse(null);
    }

    @Override
    public List<QuestionCategory> findSubsByParentId(String id) {
        return questionCategoryRepository.findByParentId(id);
    }

    @Override
    public List<QuestionCategory> getRootQuestionCategories() {
        return mongoTemplate.find(Query.query(Criteria.where("parentId").is(null)), QuestionCategory.class);
    }

    @Override
    public QuestionCategory createQuestionCategory(QuestionCategory questionCategory) {
        return questionCategoryRepository.insert(questionCategory);
    }

    @Override
    public boolean deleteCategory(QuestionCategory category) {
        questionCategoryRepository.delete(category);
        return true;
    }

    @Override
    public QuestionCategory updateQuestionCategory(QuestionCategory category) {
        QuestionCategory categoryInDb = getCategoryInfoById(category.getId());
        if (category.getName() != null) {
            categoryInDb.setName(category.getName());
        }
        if (category.getParentId() != null) {
            categoryInDb.setParentId(category.getParentId());
        }
        return questionCategoryRepository.save(categoryInDb);
    }

    @Override
    public boolean addQuestionToCategory(Question question, QuestionCategory category) {
        CategoryToQuestion mapping = new CategoryToQuestion();
        mapping.setCategoryId(category.getId());
        mapping.setQuestionId(question.getId());
        categoryToQuestionRepository.insert(mapping);
        return true;
    }

    @Override
    public boolean removeQuestionFromCategory(Question question, QuestionCategory category) {
        CategoryToQuestion categoryToQuestion = categoryToQuestionRepository
                .findByCategoryIdAndQuestionId(category.getId(), question.getId());
        categoryToQuestionRepository.delete(categoryToQuestion);
        return true;
    }
}
