package com.jtxt.demo.service;

import com.jtxt.demo.model.Question;
import com.jtxt.demo.model.QuestionCategory;
import com.jtxt.demo.repository.QuestionRepository;
import com.jtxt.demo.util.CacheStore;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;
@Service
public class QuestionServiceImpl implements QuestionService {
    @Autowired
    QuestionRepository questionRepository;
    @Autowired
    private CacheStore<Question> questionCache;
    @Override
    public Question createQuestion(Question question) {
        return questionRepository.insert(question);
    }

    @Override
    public Question editQuestion(Question updatedQuestion) {
        Question questionInDb = questionRepository.findById(updatedQuestion.getId()).orElse(null);
        if (updatedQuestion.getUserId() != null) {
            questionInDb.setUserId(updatedQuestion.getUserId());
        }
        if (updatedQuestion.getAnswers() != null) {
            questionInDb.setAnswers(updatedQuestion.getAnswers());
        }
        if (updatedQuestion.getContent() != null) {
            questionInDb.setContent(updatedQuestion.getContent());
        }
        if (updatedQuestion.getFinalAnswer() != null) {
            questionInDb.setFinalAnswer(updatedQuestion.getFinalAnswer());
        }
        if (updatedQuestion.getStatus() != null) {
            questionInDb.setStatus(updatedQuestion.getStatus());
        }
        if (updatedQuestion.getType() != null) {
            questionInDb.setType(updatedQuestion.getType());
        }
        if (updatedQuestion.getAnswerAnalysis() != null) {
            questionInDb.setAnswerAnalysis(updatedQuestion.getAnswerAnalysis());
        }
        return questionRepository.save(questionInDb);
    }

    @Override
    public boolean deleteQuestion(Question question) {
        questionRepository.delete(question);
        return true;
    }

    @Override
    public Question findQuestionById(String id) {
        Question question = questionCache.get(id);
        if (question == null) {
            question = questionRepository.findById(id).orElse(null);
            if (question != null) questionCache.add(id, question);
        }
        return question;
    }

    @Override
    public List<Question> showAllQuestions() {
        return questionRepository.findAll();
    }

    @Override
    public List<Question> showAllQuestionsUnderCategory(QuestionCategory category) {
        // TODO
        return null;
    }
}
