package com.jtxt.demo.service;

import com.jtxt.demo.model.*;
import com.jtxt.demo.repository.CommentRepository;
import com.jtxt.demo.repository.FavoriteRepository;
import com.jtxt.demo.repository.GroupToUserRepository;
import com.jtxt.demo.repository.UserRepository;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.mongodb.core.MongoTemplate;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;

import static com.jtxt.demo.util.Constants.CHINA_ZONE;

@Service
public class UserServiceImpl implements UserService {

    @Autowired
    private UserRepository userRepository;
    @Autowired
    private FavoriteRepository favoriteRepository;
    @Autowired
    private LearningMaterialService learningMaterialService;
    @Autowired
    private MongoTemplate mongoTemplate;
    @Autowired
    private GroupToUserRepository groupToUserRepository;

    @Override
    public boolean createUser(User user) {
        user.setRegisterTime(LocalDateTime.now(CHINA_ZONE));
        userRepository.insert(user);
        return true;
    }
    @Override
    public boolean createUsers(List<User> users) {
        users.stream().forEach(u -> createUser(u));
        return true;
    }
    @Override
    public boolean loginUser(User user) {
        user.setLastLoginTime(LocalDateTime.now(CHINA_ZONE));
        userRepository.save(user);
        return true;
    }
    @Override
    public boolean logoutUser(User user) {
        // nothing to do
        return true;
    }
    @Override
    public User findByUserName(String username) {
        return userRepository.findByUserName(username);
    }

    @Override
    public boolean updateUserPoints(String userName, int delta) {
        if (delta == 0) return true; // do nothing
        User userInDb = findByUserName(userName);
        userInDb.setPoints(userInDb.getPoints() + delta);
        userRepository.save(userInDb);
        return true;
    }

    @Override
    public int getTotalPoints(String userName) {
        User userInDb = findByUserName(userName);
        if (userInDb == null) return 0;
        return userInDb.getPoints();
    }

    @Override
    public User updateUser(User user) {
        User userInDb = findByUserName(user.getUserName());
        if (user.getBirthday() != null) {
            userInDb.setBirthday(user.getBirthday());
        }
        if (user.getEmail() != null) {
            userInDb.setEmail(user.getEmail());
        }
        if (user.getFirstName() != null) {
            userInDb.setFirstName(user.getFirstName());
        }
        if (user.getLastName() != null) {
            userInDb.setLastName(user.getLastName());
        }
        if (user.getPassword() != null) {
            userInDb.setPassword(user.getPassword());
        }
        if (user.getPoints() != 0) {
            userInDb.setPoints(user.getPoints());
        }
        if (user.getNickName() != null) {
            userInDb.setNickName(user.getNickName());
        }
        if (user.getPhone() != null) {
            userInDb.setNickName(user.getPhone());
        }
        if (user.getStudyId() != null) {
            userInDb.setStudyId(user.getStudyId());
        }
        return userRepository.save(userInDb);
    }

    @Override
    public void deleteUser(String userName) {
        userRepository.deleteByUserName(userName);
    }

    @Override
    public List<User> findAllUsers() {
        return userRepository.findAll();
    }
    @Autowired
    private CommentRepository commentRepository;

    @Override
    public Comment createComment(Comment comment) {
        return commentRepository.insert(comment);
    }

    @Override
    public boolean deleteComment(Comment comment) {
        commentRepository.delete(comment);
        return true;
    }

    @Override
    public boolean editComment(Comment updatedComment) {
        commentRepository.save(updatedComment);
        return true;
    }

    @Override
    public boolean addFavorite(LearningMaterial material, User user) {
        Favorite favorite = favoriteRepository.findById(user.getUserName()).orElse(new Favorite(user.getUserName()));
        List<String> materials = favorite.getMaterials();
        materials.add(material.getId());
        favoriteRepository.save(favorite);
        return true;
    }

    @Override
    public boolean removeFavorite(LearningMaterial material, User user) {
        Favorite favorite = favoriteRepository.findById(user.getUserName()).orElse(null);
        if (favorite == null) return false;
        List<String> materials = favorite.getMaterials();
        materials.remove(material.getId());
        favoriteRepository.save(favorite);
        return true;
    }

    @Override
    public Favorite showFavorites(User user) {
        Favorite favorite = favoriteRepository.findById(user.getUserName()).orElse(new Favorite(user.getUserName()));
        List<String> materialTitles = new ArrayList<>();
        favorite.getMaterials().forEach(mid -> materialTitles.add(learningMaterialService.getMaterialById(mid).getName()));
        favorite.setTitles(materialTitles);
        return favorite;
    }

    @Override
    public boolean addUserToGroup(User user, Group group) {
        GroupToUser groupToUser = new GroupToUser();
        groupToUser.setGroupId(group.getId());
        groupToUser.setUserId(user.getUserName());
        groupToUserRepository.insert(groupToUser);
        return true;
    }

    @Override
    public boolean removeUserFromGroup(User user, Group group) {
        GroupToUser mapping = groupToUserRepository.findByGroupIdAndUserId(group.getId(), user.getUserName());
        if (mapping == null) return false;
        groupToUserRepository.delete(mapping);
        return true;
    }
}
